mod benchmarks;
mod stress;
mod text;

use clap::{Args, Parser, Subcommand};

#[derive(Parser)]
#[clap(name = "Bit Goblin Benchmark", author, version, about = "Bit Goblin's hardware benchmarking tool.", long_about = None)]
#[clap(propagate_version = true)]
struct Cli {
    #[clap(subcommand)]
    command: Commands,

    #[clap(short = 'i', long, default_value_t = 1, help = "Number of times to run test. Default = 1", global = true)]
    iterations: u8,
}

#[derive(Subcommand)]
enum Commands {
    // CPU benchmarks subcommand
    #[clap(name = "cpu", about = "CPU benchmarks and stress tests.")]
    Cpu(Cpu),
    // disk benchmarks subcommand
    #[clap(name = "disk", about = "Hard drive and SSD benchmarks.")]
    Disk(Disk),
    // network benchmarks subcommand
    #[clap(name = "network", about = "Test various aspects of your network.")]
    Net(Net),
}

#[derive(Args)]
struct Cpu {
    #[clap(subcommand)]
    cpu_commands: CpuCommands,
}

#[derive(Subcommand)]
enum CpuCommands {
    // CPU stress test subcommand
    #[clap(name = "stress", about = "Stress test the CPU with math!")]
    StressTest {
        #[clap(short = 'r', long, default_value_t = 5, help = "Length of time (in minutes) to run the stress test. Defaults to 5")]
        runtime: u16,
        #[clap(short = 't', long, default_value_t = 0, help = "Number of threads to use; defaults to CPU's max thread count. Defaults to 0 (automatic)")]
        threads: usize,
    },
}

#[derive(Args)]
struct Disk {
    #[clap(subcommand)]
    disk_commands: DiskCommands,
}

#[derive(Subcommand)]
enum DiskCommands {
    // sequential disk read subcommand
    #[clap(name = "read_seq", about = "Sequential disk read speed test.")]
    ReadSeqTest {
        #[clap(short = 't', long, default_value_t = String::from("/tmp/disk-test.tmp"))]
        tempfile: String,
        #[clap(short = 's', long, default_value_t = 15)]
        size: u8,
    },

    // random disk read subcommand
    #[clap(name = "read_rand", about = "Random 4K disk read speed test.")]
    ReadRandTest {
        #[clap(short = 't', long, default_value_t = String::from("/tmp/disk-test.tmp"))]
        tempfile: String,
        #[clap(short = 's', long, default_value_t = 15)]
        size: u8,
    },

    // sequential disk write subcommand
    #[clap(name = "write_seq", about = "Write a large file to determine sequential disk write speeds.")]
    WriteSeqTest {
        #[clap(short = 't', long, default_value_t = String::from("/tmp/disk-test.tmp"))]
        tempfile: String,
        #[clap(short = 's', long, default_value_t = 15)]
        size: u8,
    },

    // random 4K disk write subcommand
    #[clap(name = "write_rand", about = "Write a bunch of smallfiles to determine random disk write speeds.")]
    WriteRandTest {
        #[clap(short = 't', long, default_value_t = String::from("/tmp/disk-test.tmp"))]
        tempfile: String,
        #[clap(short = 's', long, default_value_t = 15)]
        size: u8,
    },
}

#[derive(Parser)]
struct Net {
    #[structopt(subcommand)]
    net_commands: NetCommands,
}

#[derive(Subcommand)]
enum NetCommands {
    // ping subcommand
    #[clap(name = "ping", about = "Ping a host to determine network latency.")]
    Ping {
        #[clap(short = 'a', long, default_value_t = String::from("8.8.8.8"))]
        address: String,
        #[clap(short = 'c', long, default_value_t = 30)]
        count: u16,
    },

    // jitter subcommand
    #[clap(name = "jitter", about = "Ping a host to determine network jitter.")]
    Jitter {
        #[clap(short = 'a', long, default_value_t = String::from("8.8.8.8"))]
        address: String,
        #[clap(short = 'c', long, default_value_t = 30)]
        count: u16,
    },

    // bandwidth test subcommand
    #[clap(name = "bandwidth", about = "Downloads a remote file to determine network bandwidth.")]
    Bandwidth {
        #[clap(short = 'd', long, default_value_t = String::from("https://www.bitgoblin.tech/hardware-tests/export-01.mp4"))]
        download: String,
        #[clap(short = 't', long, default_value_t = String::from("/tmp/bw-test.tmp"))]
        output: String,
    },
}

fn main() {
    let cli = Cli::parse();

    // map subcommands back to the main command
    match &cli.command {
        Commands::Cpu(args) => match &args.cpu_commands {
            CpuCommands::StressTest { runtime, threads } => stress::cpu::cpu_stress_math(*runtime, *threads),
        },

        Commands::Disk(args) => match &args.disk_commands {
            DiskCommands::ReadSeqTest { tempfile, size } => {
                for i in 0..cli.iterations {
                    println!("Test run number {}.", i + 1);
                    benchmarks::disk::disk_read_seq_test(tempfile, size);
                }
            },
            DiskCommands::ReadRandTest { tempfile, size } => {
                for i in 0..cli.iterations {
                    println!("Test run number {}.", i + 1);
                    benchmarks::disk::disk_read_rand_test(tempfile, size);
                }
            },
            DiskCommands::WriteSeqTest { tempfile, size } => {
                for i in 0..cli.iterations {
                    println!("Test run number {}.", i + 1);
                    benchmarks::disk::disk_write_seq_test(tempfile, size);
                }
            },
            DiskCommands::WriteRandTest { tempfile, size } => {
                for i in 0..cli.iterations {
                    println!("Test run number {}.", i + 1);
                    benchmarks::disk::disk_write_rand_test(tempfile, size);
                }
            },
        }

        Commands::Net(args) => match &args.net_commands {
            NetCommands::Ping { address, count } => {
                for i in 0..cli.iterations {
                    println!("Test run number {}.", i + 1);
                    benchmarks::network::ping_host(address, count);
                }
            },
            NetCommands::Jitter { address, count } => {
                for i in 0..cli.iterations {
                    println!("Test run number {}.", i + 1);
                    benchmarks::network::jitter_test(address, count);
                }
            },
            NetCommands::Bandwidth { download, output } => {
                for i in 0..cli.iterations {
                    println!("Test run number {}.", i + 1);
                    benchmarks::network::bandwidth_test(download, output);
                }
            },
        },
    }
}
